@@ -4,6 +4,7 @@ import shutil
 import argparse 
 import textwrap
 import json
+import parallel_driver
 
 class Open_With_Indent:
 
@@ -33,9 +34,12 @@ class Experiment_Setup:
         if 'years' in dir(self):
             self.years=self.years.split(',')
 
+        #For parallel option:
+        self.parallel_dimension=None
+        self.parallel_dimension_number=0
+
     def prepare_scripts(self):
         """ Prepares the scripts for bash launch """
-        parallel_dimension_number=0
     
         runscript_file=self.runscripts_dir+'/'+'_'.join([self.diagnostic,
                                                             self.model,
@@ -44,7 +48,6 @@ class Experiment_Setup:
                                                             self.experiment,
                                                             '-'.join(self.years)])
         out=Open_With_Indent(runscript_file,'w')
-        #out=Open_With_Indent(self.diagnostics_dir+'/'+self.diag_name+'.sh','w')
 
         out.writei('#!/bin/bash\n')
         if self.pbs_expt:
@@ -52,8 +55,8 @@ class Experiment_Setup:
             #out.writei('#PBS -l nodes=1:ppn={0},walltime={1}\n'.format(max(self.dim_async,self.m_async),self.walltime))
             out.writei('#PBS -l nodes=1:ppn={0}\n'.format(max(self.dim_async,self.m_async)))
             out.writei('#PBS -N {0}_{1}_{2}_{3}_{4}\n'.format(self.years[0],self.years[1],self.model,self.run_id,self.experiment))
-            out.writei('#PBS -o {5}/pbs_out/{0}_{1}_{2}_{3}_{4}\n'.format(self.years[0],self.years[1],self.model,self.run_id,self.experiment,self.source_dir))
-            out.writei('#PBS -e {5}/pbs_err/{0}_{1}_{2}_{3}_{4}\n'.format(self.years[0],self.years[1],self.model,self.run_id,self.experiment,self.source_dir))
+            out.writei('#PBS -o {5}/pbs_out/{0}_{1}_{2}_{3}_{4}\n'.format(self.years[0],self.years[1],self.model,self.run_id,self.experiment,self.output_dir))
+            out.writei('#PBS -e {5}/pbs_err/{0}_{1}_{2}_{3}_{4}\n'.format(self.years[0],self.years[1],self.model,self.run_id,self.experiment,self.output_dir))
 
         out.writei('\n')
         out.writei('#The next variable should be empty. If your script completed and some\n')
@@ -72,14 +75,14 @@ class Experiment_Setup:
         out.writei('export CDB_OUT_FILE=${CDB_DIAG_NAME}_${CDB_MODEL}_${CDB_EXPT}_$(echo $CDB_YEARS | tr \',\' \'_\')_${CDB_RUN_ID}\n')
         out.writei('\n')
         out.writei('#SET THE OUTPUT DIRECTORY\n')
-        out.writei('CDB_OUT_DIR="{0}"\n'.format(self.output_dir))
+        out.writei('export CDB_OUT_DIR="{0}"\n'.format(self.output_dir))
         out.writei('mkdir -p ${CDB_OUT_DIR}\n')
         out.writei('\n')
         out.writei('#SET THE TEMP DIRECTORY\n')
         out.writei('CDB_ROOT_TEMP_DIR="{0}"\n'.format(self.temp_dir))
         out.writei('if [ -z "${PBS_JOBID}" ]; then\n')
         out.inc_indent()
-        out.writei('export CDB_TEMP_DIR="${CDB_ROOT_TEMP_DIR}/${CDB_DIAG_NAME}.jod_pid${$}"\n')
+        out.writei('export CDB_TEMP_DIR="${CDB_ROOT_TEMP_DIR}/${CDB_DIAG_NAME}.job_pid${$}"\n')
         out.dec_indent()
         out.writei('else\n')
         out.inc_indent()
@@ -90,7 +93,7 @@ class Experiment_Setup:
         out.writei('\n')
         out.writei('#Subset the data pointers file and put it in temporary directory for optimal retrieval:\n')
         out.writei('CDB_DIAG_HEADER_BASE=$(basename ${CDB_DIAG_HEADER%.gz})\n')
-        out.writei('cdb_query_archive slice --experiment=${CDB_EXPT} --model=${CDB_MODEL} --center=${CDB_CENTER} --rip=${CDB_RUN_ID} ${CDB_DIAG_HEADER} ${CDB_TEMP_DIR}/${CDB_DIAG_HEADER_BASE}\n')
+        out.writei('cdb_query_archive slice --experiment=${CDB_EXPT} --model=${CDB_MODEL} --center=${CDB_CENTER} ${CDB_DIAG_HEADER} ${CDB_TEMP_DIR}/${CDB_DIAG_HEADER_BASE}\n')
         out.writei('CDB_DIAG_HEADER="${CDB_TEMP_DIR}/${CDB_DIAG_HEADER_BASE}"\n')
 
         
@@ -98,179 +101,105 @@ class Experiment_Setup:
         script_file=open(self.diagnostic_dir+'/'+self.diagnostic+'.sh','r')
 
         parallel_dimension=None
-        instructions={'#!START MONTH LOOP' : start_monthly_loop,
+        instructions={
+                      '#!START MONTH LOOP' : start_monthly_loop,
                       '#!END MONTH LOOP' : end_monthly_loop,
-                      }
-
+                      '#!PARALLEL' : parallel_driver.start_parallel_instance,
+                      '#!END PARA' : parallel_driver.end_parallel_instance
+                    }
 
         for line in script_file:
-            if line.lstrip()[:10].upper() == '#!PARALLEL' and self.dim_async > 1:
-                if parallel_dimension != None:
-                    raise IOError('Attempting to parallelize a dimension without having closed the previous parallel instance')
-                else:
-                    parallel_dimension = line.lstrip()[10:].rstrip('\n').strip()
-                out.writei('#Creating a parallel loop over dimension '+parallel_dimension+' with '+str(self.dim_async)+' processors:\n')
-                out.writei('#########################################################################\n')
-                out.writei('#Identify the files to transfer\n')
-                out.writei('LIST_TEMP_FILE=`ls ${CDB_TEMP_DIR}/${CDB_TEMP_FILE}*`\n')
-                out.writei('export LIST_FILE_ID=`for FILE in ${LIST_TEMP_FILE}; do echo ${FILE#${CDB_TEMP_DIR}/${CDB_TEMP_FILE}}; done`\n')
-                out.writei('export CDB_TEMP_FILE="${CDB_TEMP_FILE}"\n')
-
-                out.writei('\n')
-                out.writei('#Define splitting function:\n')
-                out.writei('function parallel_dimension_'+parallel_dimension+str(parallel_dimension_number)+'() {\n')
-
-                out.inc_indent()
-                out.writei('cat > $1 <<\'EndOfFunction\'\n')
-                out.writei('NUM_DIM=`echo $1 | tr \',\' \' \' | awk -F\' \' \'{ print $1 }\'`\n')
-                out.writei('LAST_NUM=`echo $1 | tr \',\' \' \' | awk -F\' \' \'{ print $2 }\'`\n')
-                out.writei('NUM_ID=$(printf "%03d" ${NUM_DIM})\n')
-                out.writei('for FILE_ID in $LIST_FILE_ID; do\n')
-
-                out.inc_indent()
-                out.writei('DIM_LENGTH=`ncks -H -v '+parallel_dimension+' ${CDB_TEMP_DIR}/${CDB_TEMP_FILE}${FILE_ID} | wc -l`\n')
-                out.writei('let "DIM_LENGTH-=1"\n')
-                out.writei('if [ "$DIM_LENGTH" -gt "1" ]; then\n')
-
-                out.inc_indent()
-                out.writei('ncks -d '+parallel_dimension+',${NUM_DIM},${LAST_NUM} ${CDB_TEMP_DIR}/${CDB_TEMP_FILE}${FILE_ID} ${CDB_TEMP_DIR}/${CDB_TEMP_FILE}.'+parallel_dimension+'${NUM_ID}${FILE_ID}\n')
-                out.dec_indent()
-                out.writei('else\n')
-                out.inc_indent()
-                out.writei('cp ${CDB_TEMP_DIR}/${CDB_TEMP_FILE}${FILE_ID} ${CDB_TEMP_DIR}/${CDB_TEMP_FILE}.'+parallel_dimension+'${NUM_ID}${FILE_ID}\n')
-                out.dec_indent()
-                out.writei('fi\n')
-
-                out.dec_indent()
-                out.writei('done\n')
-                out.writei('if [ -e "${CDB_TEMP_DIR}/${CDB_TEMP_FILE}" ]; then\n')
-                out.writei('ncks -d '+parallel_dimension+',${NUM_DIM},${LAST_NUM} ${CDB_TEMP_DIR}/${CDB_TEMP_FILE} ${CDB_TEMP_DIR}/${CDB_TEMP_FILE}.'+parallel_dimension+'${NUM_ID}\n')
-                out.writei('fi\n')
-                out.writei('CDB_TEMP_FILE="${CDB_TEMP_FILE}.'+parallel_dimension+'${NUM_ID}"\n')
-                out.writei('#########################################################################\n')
-                out.inc_indent()
-                out.inc_indent()
-            elif ( (line.lstrip()[:14].upper() == '#!END PARALLEL' or line.lstrip()[:14].upper() == '#!END PARAFULL') and self.dim_async > 1):
-                if line.lstrip()[14:].rstrip('\n').strip() != parallel_dimension:
-                    raise IOError('Attempting to close a different parallel instance')
-                else:
-                    out.dec_indent()
-                    out.dec_indent()
-                    out.writei('#########################################################################\n')
-                    out.writei('for FILE_ID in $LIST_FILE_ID; do rm -f ${CDB_TEMP_DIR}/${CDB_TEMP_FILE}${FILE_ID}; done\n')
-                    out.writei('if [ -e "${CDB_TEMP_DIR}/${CDB_TEMP_FILE}" ]; then\n')
-                    out.writei('rm ${CDB_TEMP_DIR}/${CDB_TEMP_FILE}\n')
-                    out.writei('fi\n')
-                    out.writei('\n')
-                    if parallel_dimension != 'time':
-                         out.writei('LIST_LOCAL_FILE=`ls ${CDB_TEMP_DIR}/${CDB_TEMP_FILE}*`\n')
-                         out.writei('export LIST_LOCAL_FILE_ID=`for FILE in ${LIST_LOCAL_FILE}; do echo ${FILE#${CDB_TEMP_DIR}/${CDB_TEMP_FILE}}; done`\n')
-                         out.writei('for FILE_ID in $LIST_LOCAL_FILE_ID; do\n')
-                         out.inc_indent()
-                         out.writei('ncpdq -a '+parallel_dimension+',time ${CDB_TEMP_DIR}/${CDB_TEMP_FILE}${FILE_ID} ${CDB_TEMP_DIR}/${CDB_TEMP_FILE}${FILE_ID}_perm\n')
-                         out.writei('mv ${CDB_TEMP_DIR}/${CDB_TEMP_FILE}${FILE_ID}_perm ${CDB_TEMP_DIR}/${CDB_TEMP_FILE}${FILE_ID}\n')
-                         out.dec_indent()
-                         out.writei('done\n')
-
-
-                    out.dec_indent()
-                    out.open.write('EndOfFunction\n')
-                    out.dec_indent()
-                    out.writei('}\n')
-                    out.writei('parallel_dimension_'+parallel_dimension+str(parallel_dimension_number)+' ${CDB_TEMP_DIR}/'+parallel_dimension+'_function'+str(parallel_dimension_number)+'.sh\n')
-                    out.writei('#Performing parallel computation\n')
-                    out.writei('DIM_LENGTHS=`for FILE in $LIST_TEMP_FILE; do ncks -H -v '+parallel_dimension+' ${FILE} | wc -l; done`\n')
-                    out.writei('DIM_LENGTH=`for DIM in ${DIM_LENGTHS}; do echo $DIM; done | awk \'{if(min==""){min=max=$1}; if($1>max) {max=$1}; if($1<min) {min=$1}} END {print max}\'`\n')
-                    out.writei('let "DIM_LENGTH -= 2"\n')
-                    if line.lstrip()[:14].upper() == '#!END PARAFULL':
-                        out.writei('STRIDE=1\n')
-                    else:
-                        out.writei('STRIDE=`expr $DIM_LENGTH / '+str(self.dim_async)+' + 1`\n')
-                    out.writei('NUM_DIM=0\n')
-                    out.writei('while [ "$NUM_DIM" -le "$DIM_LENGTH" ]; do if [ "$((${NUM_DIM} + ${STRIDE} - 1))" -ge "$DIM_LENGTH" ]; then \\\n')
-                    out.writei('echo ${NUM_DIM},${DIM_LENGTH}; else echo ${NUM_DIM},$((${NUM_DIM} + ${STRIDE} - 1)); fi; \\\n')
-                    out.writei('let "NUM_DIM += ${STRIDE}"; done |\\\n')
-                    out.writei('parallel -j'+str(self.dim_async)+' -k "bash ${CDB_TEMP_DIR}/'+parallel_dimension+'_function'+str(parallel_dimension_number)+'.sh {}"\n')
-                    out.writei('rm ${CDB_TEMP_DIR}/'+parallel_dimension+'_function'+str(parallel_dimension_number)+'.sh\n')
-                    out.writei('\n')
-                    out.writei('#Delete files that were deleted in the parallel process\n')
-                    out.writei('for FILE_ID in $LIST_FILE_ID; do\n')
-                    out.inc_indent()
-                    out.writei('if [ ! -f ${CDB_TEMP_DIR}/${CDB_TEMP_FILE}.'+parallel_dimension+'000${FILE_ID} ]; then rm ${CDB_TEMP_DIR}/${CDB_TEMP_FILE}${FILE_ID}; fi\n')
-                    out.dec_indent()
-                    out.writei('done\n')
-
-                    out.writei('#Recombine output files\n')
-                    out.writei('LIST_PARALLEL_FILE=`ls ${CDB_TEMP_DIR}/${CDB_TEMP_FILE}.'+parallel_dimension+'000*`\n')
-                    out.writei('LIST_PARALLEL_FILE_ID=`for FILE in ${LIST_PARALLEL_FILE}; do echo ${FILE#${CDB_TEMP_DIR}/${CDB_TEMP_FILE}.'+parallel_dimension+'000}; done`\n')
-
-                    out.writei('for FILE_ID in $LIST_PARALLEL_FILE_ID; do\n')
-                    out.inc_indent()
-                    out.writei('if [ ! -f ${CDB_TEMP_DIR}/${CDB_TEMP_FILE}${FILE_ID} ]; then\n')
-                    out.inc_indent()
-                    out.writei('ncrcat ${CDB_TEMP_DIR}/${CDB_TEMP_FILE}.'+parallel_dimension+'???${FILE_ID} ${CDB_TEMP_DIR}/${CDB_TEMP_FILE}${FILE_ID}\n')
-                    out.writei('rm ${CDB_TEMP_DIR}/${CDB_TEMP_FILE}.'+parallel_dimension+'???${FILE_ID}\n')
-                    if parallel_dimension != 'time':
-                        out.writei('ncpdq -a time,'+parallel_dimension+' ${CDB_TEMP_DIR}/${CDB_TEMP_FILE}${FILE_ID} ${CDB_TEMP_DIR}/${CDB_TEMP_FILE}${FILE_ID}_perm\n')
-                        out.writei('mv ${CDB_TEMP_DIR}/${CDB_TEMP_FILE}${FILE_ID}_perm ${CDB_TEMP_DIR}/${CDB_TEMP_FILE}${FILE_ID}\n')
-                    out.dec_indent()
-                    out.writei('else\n')
-                    out.inc_indent()
-                    out.writei('rm ${CDB_TEMP_DIR}/${CDB_TEMP_FILE}.'+parallel_dimension+'???${FILE_ID}\n')
-                    out.dec_indent()
-                    out.writei('fi\n')
-                    out.dec_indent()
-                    out.writei('done\n')
-                    out.writei('#########################################################################\n')
-
-                    #These parameters are important if many parallel instances are specified:
-                    parallel_dimension=None
-                    parallel_dimension_number+=1
-            else:
                 out.writei(line)
                 for inst in instructions.keys():
                     if line.lstrip().upper()[:len(inst)]==inst:
-                        instructions[inst](self,out)
-        #Finally, introduce the code to structure the output as in an ESG archive:
-        #out.writei('if [ ! -z "$CDB_CMIP5_COMP_LIST" ]; then\n')
-        #out.inc_indent()
-        #out.writei('for FILE_DESC in $CDB_CMIP5_COMP_LIST; do\n')
-        #out.inc_indent()
-        #out.writei('FILE=$(echo $FILE_DESC | awk -F\';\' \'{print $1}\')\n')
-        #out.writei('REALM_FREQ_INFO=$(echo $FILE_DESC | awk -F\';\' \'{print $2}\')\n')
-        #out.writei('bash $CDB_SOURCE_DIR/structure_out_as_in.sh -d ${CDB_DIAG_NAME} -m ${CDB_MODEL} -e ${CDB_EXPT} -r ${CDB_RUN_ID} -y ${CDB_YEARS} -f ${REALM_FREQ_INFO} $FILE\n')
-        #out.dec_indent()
-        #out.writei('done\n')
-        #out.dec_indent()
-        #out.writei('fi\n')
-        #if self.pbs_expt:
+                        self=instructions[inst](self,out,line)
+
+        out.writei('\n')
         out.writei('rm -rf ${CDB_TEMP_DIR}\n')
+        out.writei('\n')
+        structure_out_with_cmip5_drs(self,out)
         out.open.close()
 
-def start_monthly_loop(self,out):
+def structure_out_with_cmip5_drs(self,out):
+    #Finally, introduce the code to structure the output with the CMIP5 DRS:
+    out.writei('#THE LAST PART OF THIS SCRIPT REORGANIZES THE OUTPUT TO CONFORM WITH THE CMIP5 DRS:\n')
+    out.writei('if [ ! -z "$CDB_CMIP5_COMP_LIST" ]; then\n')
+    out.inc_indent()
+    out.writei('for FILE_DESC in $CDB_CMIP5_COMP_LIST; do\n')
+    out.inc_indent()
+    out.writei('FILE=$(echo $FILE_DESC | awk -F\';\' \'{print $1}\')\n')
+    out.writei('REALM_FREQ_INFO=$(echo $FILE_DESC | awk -F\';\' \'{print $2}\')\n')
+    out.writei('bash $CDB_SOURCE_DIR/structure_out_as_in.sh -d ${CDB_DIAG_NAME} -m ${CDB_MODEL} -e ${CDB_EXPT} -r ${CDB_RUN_ID} -y ${CDB_YEARS} -f ${REALM_FREQ_INFO} $FILE\n')
+    out.writei('CDB_INFO="This output was created using the Climate Diagnostics Benchmark toolbox,\
+                developed by F. Laliberte and P. J. Kushner, 2012, at the University of Toronto.\
+                This code was developed as part of the ExArch project."\n')
+    out.writei('\n')
+    out.writei('CDB_VAR_LIST=$(cdo showname ${FILE})\n')
+    out.writei('\n')
+    out.writei('CDB_FREQ=`echo ${REALM_FREQ_INFO} | tr \',\' \' \' | awk -F\' \' \'{ print $1}\'`\n')
+    out.writei('CDB_REALM=`echo ${REALM_FREQ_INFO} | tr \',\' \' \' | awk -F\' \' \'{ print $2}\'`\n')
+    out.writei('CDB_MIP=`echo ${REALM_FREQ_INFO} | tr \',\' \' \' | awk -F\' \' \'{ print $3}\'`\n')
+    out.writei('\n')
+    out.writei('for CDB_VAR_NAME in $CDB_VAR_LIST; do\n')
+    out.inc_indent()
+    out.writei('CDB_LOCAL_RUN_ID={0}\n'.format(self.run_id))
+    out.writei('if [ "$CDB_FREQ" == "fx" ]; then\n')
+    out.inc_indent()
+    out.writei('if [ "$CDB_MIP" == "fx" ]; then\n')
+    out.inc_indent()
+    out.writei('CDB_LOCAL_RUN_ID="r0i0p0"\n')
+    out.dec_indent()
+    out.writei('fi\n')
+    out.dec_indent()
+    out.writei('fi\n')
+    out.writei('BASE_PATH="$CDB_OUT_DIR/in/{0}/{1}/{2}/$CDB_FREQ/$CDB_REALM/$CDB_MIP/$CDB_LOCAL_RUN_ID/v$(date +%F | tr -d \'-\')/$CDB_VAR_NAME"\n'.format(self.center,self.model,self.experiment))
+    out.writei('mkdir -p $BASE_PATH\n')
+    out.writei('FILE_NAME="$BASE_PATH/${CDB_VAR_NAME}_${CDB_MIP}_'+self.model+'_'+self.experiment+'_${CDB_LOCAL_RUN_ID}"\n')
+    out.writei('case $CDB_FREQ in\n')
+    out.inc_indent()
+    out.writei('fx) FILE_NAME="${FILE_NAME}";;\n')
+    out.writei('day) FILE_NAME="${FILE_NAME}_${CDB_YEAR_START}0101-${CDB_YEAR_END}1231";;\n')
+    out.writei('mon) FILE_NAME="${FILE_NAME}_${CDB_YEAR_START}01-${CDB_YEAR_END}12";;\n')
+    out.writei('yr) FILE_NAME="${FILE_NAME}_${CDB_YEAR_START}-${CDB_YEAR_END}";;\n')
+    out.writei('*) echo "Unrecognized output frequency $CDB_FREQ";;\n')
+    out.dec_indent()
+    out.writei('esac\n')
+    out.writei('cdo -s -f nc4 -z zip -selname,${CDB_VAR_NAME} $FILE ${FILE_NAME}.nc\n')
+    #out.writei('echo $CDB_INFO > ${FILE_NAME}.info\n')
+    #out.writei('cdo -s -f nc4 -z zip -setgatt,CDB_info,${FILE_NAME}.info -selname,${CDB_VAR_NAME} $FILE ${FILE_NAME}.nc\n')
+    #out.writei('rm ${FILE_NAME}.info\n')
+    out.dec_indent()
+    out.writei('done\n')
+    out.writei('rm $FILE\n')
+    out.dec_indent()
+    out.writei('done\n')
+    out.dec_indent()
+    out.writei('fi\n')
+
+def start_monthly_loop(self,out,line):
     out.writei('function monthly_processing()\n')
     out.writei('{\n')
     out.writei('cat > $3 <<EndOfScriptHeader\n')
     out.writei('#!/bin/bash\n')
     out.writei('#This file launches a monthly diagnostic\n')
-    out.writei('CDB_YEAR=$1\n')
-    out.writei('CDB_MONTH=$2\n')
+    out.writei('export CDB_YEAR=$1\n')
+    out.writei('export CDB_MONTH=$2\n')
     out.writei('\n')
     #out.writei('CDB_YEAR_START=${CDB_YEAR_START}\n')
     #out.writei('CDB_YEAR_END=${CDB_YEAR_END}\n')
-    out.writei('CDB_CENTER=${CDB_CENTER}\n')
-    out.writei('CDB_MODEL=${CDB_MODEL}\n')
-    out.writei('CDB_RUN_ID=${CDB_RUN_ID}\n')
-    out.writei('CDB_EXPT=${CDB_EXPT}\n')
-    out.writei('CDB_DIAG_NAME=${CDB_DIAG_NAME}\n')
-    out.writei('CDB_DIAG_HEADER=${CDB_DIAG_HEADER}\n')
-    out.writei('CDB_TEMP_DIR=${CDB_TEMP_DIR}\n')
-    out.writei('CDB_OUT_FILE=${CDB_OUT_FILE}_${CDB_YEAR}_${CDB_MONTH}\n')
+    out.writei('export CDB_CENTER=${CDB_CENTER}\n')
+    out.writei('export CDB_MODEL=${CDB_MODEL}\n')
+    out.writei('export CDB_RUN_ID=${CDB_RUN_ID}\n')
+    out.writei('export CDB_EXPT=${CDB_EXPT}\n')
+    out.writei('export CDB_DIAG_NAME=${CDB_DIAG_NAME}\n')
+    out.writei('export CDB_DIAG_HEADER=${CDB_DIAG_HEADER}\n')
+    out.writei('export CDB_TEMP_DIR=${CDB_TEMP_DIR}\n')
+    out.writei('export CDB_OUT_FILE=${CDB_OUT_FILE}_${CDB_YEAR}_${CDB_MONTH}\n')
     #for type_var in cdb_environment_variables.keys():
     #    for name_var in cdb_environment_variables[type_var]:
     #        out.writei('export CDB_{0}_{1}=$CDB_{0}_{1}\n'.format(name_var.upper(),type_var.upper()))
     out.writei('\n')
-    out.writei('CDB_VAR_LIST="${CDB_VAR_LIST}"\n')
+    out.writei('export CDB_VAR_LIST="${CDB_VAR_LIST}"\n')
 
     out.open.write('EndOfScriptHeader\n')
     out.writei('\n')
@@ -278,16 +207,19 @@ def start_monthly_loop(self,out):
     out.writei('\n')
     out.writei('#Retrieve path names to data:\n')
     for var_name in self.variable_list.keys():
-        retrieval_string='${CDB_CENTER} ${CDB_MODEL} ${CDB_EXPT} ${CDB_RUN_ID} '
+        if self.variable_list[var_name][0]!='fx':
+            retrieval_string='${CDB_CENTER} ${CDB_MODEL} ${CDB_EXPT} ${CDB_RUN_ID} '
+        else:
+            retrieval_string='${CDB_CENTER} ${CDB_MODEL} ${CDB_EXPT} r0i0p0 '
         retrieval_string+=var_name+' '
         retrieval_string+=' '.join(self.variable_list[var_name])
         retrieval_string+=' ${CDB_YEAR} ${CDB_MONTH}'
         retrieval_string+=' ${CDB_DIAG_HEADER}'
-        out.writei('CDB_'+var_name+'_'+'_'.join(self.variable_list[var_name])+'=$(cdb_query_archive retrieve '+retrieval_string+')\n')
-        out.writei('CDB_'+var_name+'_'+'_'.join(self.variable_list[var_name])+'_TYPE=$(cdb_query_archive retrieve -f '+retrieval_string+')\n')
+        out.writei('export CDB_'+var_name+'_'+'_'.join(self.variable_list[var_name])+'=$(cdb_query_archive retrieve '+retrieval_string+')\n')
+        out.writei('export CDB_'+var_name+'_'+'_'.join(self.variable_list[var_name])+'_TYPE=$(cdb_query_archive retrieve -f '+retrieval_string+')\n')
     out.writei('\n')
     out.writei('#Create a cdo retrieval script:\n')
-    out.writei('CDO_RETRIEVAL_SCRIPT="cdo merge "\n')
+    out.writei('CDO_RETRIEVAL_SCRIPT=""\n')
     for var_name in self.variable_list.keys():
         var_id = '$CDB_'+var_name+'_'+'_'.join(self.variable_list[var_name])
         out.writei('#Creating retrieval for {0}\n'.format(var_name))
@@ -296,23 +228,9 @@ def start_monthly_loop(self,out):
         out.writei('CDB_VAR_END=$(echo '+var_id+' | awk -F\'|\' \'{print $3}\')\n')
         out.writei('CDB_TIME_STEPS=$(echo $(seq $((CDB_VAR_START+1)) $((CDB_VAR_END+1))) | tr \' \' \',\')\n')
         out.writei('CDO_RETRIEVAL_SCRIPT="${CDO_RETRIEVAL_SCRIPT} -selname,'+var_name+' -seltimestep,${CDB_TIME_STEPS} $CDB_VAR"\n')
-        
+    return self
 
-
-        #if self.time_diff:
-        #    retrieval_string='${CDB_CENTER} ${CDB_MODEL} ${CDB_EXPT} ${CDB_RUN_ID} '
-        #    retrieval_string+=var_name+' '
-        #    retrieval_string+=' '.join(self.variable_list[var_name])
-        #    out.writei('#Previous month\n')
-        #    out.writei('if [ "${CDB_MONTH}" -eq "1" ]; then\n')
-        #    out.writei('let "CDB_YEAR-=1"\n')
-        #    out.writei('CDB_'+var_name+'_'+'_'.join(self.variable_list[var_name])+'_PREV=$('+retrieval_string+')\n')
-        #    out.writei('#Next month\n')
-        #    out.writei('let "CDB_YEAR+=2"\n')
-        #    out.writei('CDB_'+var_name+'_'+'_'.join(self.variable_list[var_name])+'_NEXT=$('+retrieval_string+')\n')
-        #    out.writei('let "CDB_YEAR-=1"\n')
-
-def end_monthly_loop(self,out):
+def end_monthly_loop(self,out,line):
     out.open.write('EndOfScriptMain\n')
     out.writei('}\n')
 
@@ -358,10 +276,11 @@ def end_monthly_loop(self,out):
     elif self.m_async == 1:
         out.writei('for SCRIPT in $(ls ${CDB_TEMP_DIR}/script_????_??.sh); do bash $SCRIPT > ${CDB_OUT_DIR}/${CDB_OUT_FILE}.out 2>&1; rm $SCRIPT; done\n')
     else:
-        out.writei('ls ${CDB_TEMP_DIR}/script_????_??.sh | parallel --tmpdir ${CDB_OUT_DIR} -k -j'+str(self.m_async)+' "bash {}" > ${CDB_OUT_DIR}/${CDB_OUT_FILE}.out 2>&1 &\n')
+        out.writei('ls ${CDB_TEMP_DIR}/script_????_??.sh | parallel --tmpdir ${CDB_OUT_DIR} -k -j'+str(self.m_async)+' "bash {}; rm {}" > ${CDB_OUT_DIR}/${CDB_OUT_FILE}.out 2>&1 &\n')
         out.writei('pid=${!}\n')
         out.writei('${CDB_SOURCE_DIR}/parallel_ramdisk.sh $pid ${CDB_TEMP_DIR}\n')
         out.writei('rm ${CDB_TEMP_DIR}/script_????_??.sh\n')
+    return self
 
 def main():
 
@@ -476,10 +395,13 @@ def main():
                 experiment = Experiment_Setup(options)
                 experiment.prepare_scripts()
 
-                #if options.run:
-                #
-                #elif options.submit:
+                if options.run:
+                    print 'Not implemented yet'
+                elif options.submit:
+                    print 'Not implemented yet'
 
+    if not options.run and not options.submit:
+        print 'Scripts are available in '+options.runscripts_dir
 
 if __name__ == "__main__":
     main()
